#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os
import copy
import time
import argparse


import technoap.sitcp as sitcp
import technoap.device.apu101 as apu101
import technoap.time.stopwatch as stopwatch

	
#device IP address
DEST_IPADDR = "192.168.10.128"

##########################################################
def main(argv):

	print ("start monitoring HV status...")

	try:
		#create 'RBCP' instance
		rbcp = sitcp.rbcp.RBCP(True)
		rbcp.setdefaultdest(DEST_IPADDR)

		#for high voltage
		status = apu101.statusHV.StatusHV(rbcp)

		########################################################
		monitorSec = argv.mon_sec
		intervalSec = argv.interval
		watching = stopwatch.Stopwatch()
		watching.start()

		while True:
			elapsedSec = watching.getElapsedSecond()
			readed = status.readStatusAll()		#status read

			if readed == False or elapsedSec >= monitorSec:
				break;

			########################################
			him = status.toPhysicalQuantityHIM(status.mStatus["HIM",])	#current(micro A)
			hvm = status.toPhysicalQuantityHVM(status.mStatus["HVM",])	#volt (monitor)
			hvd = status.toPhysicalQuantityHVD(status.mStatus["HVD",])	#volt (setting)
			hsm = status.toPhysicalQuantityHSM(status.mStatus["HSM",])	#bias shutdown volt

			print ("\n########## %.1fs / %.1fs ##########" % (elapsedSec, monitorSec))
			print ("em:%d, pol:%d, sweep:%d, bias-shtd:%d" % (status.mStatusEmergency, status.mStatusPolarity, status.mStatusSweep, status.mStatusBiasShutdown))
			print (" %.1f / %.1f(V), %.1f(uA), %.1f(bias.V)" % (hvm, hvd, him, hsm))
			########################################
			time.sleep(intervalSec)

		watching.stop()
		########################################################

	except Exception as ex:
		print (ex)

	print ("monitoring HV status complete!!!")

##########################################################
if __name__ == "__main__":
	parser = argparse.ArgumentParser(description='monitoring status for HV')
	parser.add_argument("--time",     dest="mon_sec",  type=float, default=30.0, help="monitoring counts")
	parser.add_argument("--interval", dest="interval", type=float, default=1.0, help="monitoring interval(sec)")
	args = parser.parse_args()
	main(args)

