#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys


############################################################
TIME_DIGIT2NANOSEC = 10     #10ns/digit


############################################################
##### common register ######################################
############################################################
##### MOD: measurement mode #####
REG_MOD_HISTOGRAM = 0
REG_MOD_WAVE = 7

##### MMD: measurement time mode #####
REG_MMD_REALTIME = 0
REG_MMD_LIVETIME = 1

##### AQS: acquisition start / stop #####
REG_AQS_STOP  = 0
REG_AQS_START = 1

##### CLR: clear #####
REG_CLR_LOW  = 0
REG_CLR_HIGH = 1

##### CLS: clock source #####
REG_CLS_INTERNAL = 0    #only this parameter value is supported
REG_CLS_EXTERNAL = 1    #no support
REG_CLS_DEFAULT  = REG_CLS_INTERNAL     #use default value

##### SCS: sampling rate(wave mode) #####
REG_SCS_10NS = 0    #100MHz, only this parameter value is supported
REG_SCS_20NS = 1    #50MHz
REG_SCS_40NS = 2    #25MHz
REG_SCS_80NS = 3    #12.5MHz
REG_SCS_DEFAULT = REG_SCS_10NS      #use default value

##### FRN: free run(wave mode) #####
REG_FRN_OFF = 0     #
REG_FRN_ON  = 1     #

##### TSO: trigger source(wave mode) #####
REG_TSO_SIG1 = 0    #CH1
REG_TSO_SIG2 = 1    #CH2, no support
REG_TSO_SIG3 = 2    #CH3, no support
REG_TSO_SIG4 = 3    #CH4, no support
REG_TSO_SIG5 = 4    #CH5, no support
REG_TSO_SIG6 = 5    #CH6, no support
REG_TSO_SIG7 = 6    #CH7, no support
REG_TSO_SIG8 = 7    #CH8, no support
REG_TSO_DEFAULT = REG_TSO_SIG1      #use default value

##### TLV: trigger level(wave mode) #####
REG_TLV_MIN = -8192
REG_TLV_MAX =  8192

##### TPO: trigger position(wave mode) #####
REG_TPO_MIN =    0
REG_TPO_MAX = 2000

REG_TPO_MIND =    0
REG_TPO_MAXD = 1023

##### TGE: trigger edge(wave mode) #####
REG_TGE_NEG = 0     #negative
REG_TGE_POS = 1     #positive


##### DAC: DAC monitor type #####
REG_DAC_PREAMP = 0
REG_DAC_FAST   = 1
REG_DAC_SLOW   = 2
REG_DAC_CFD    = 3

##### RCH: ROI SCA-CH #####
REG_RCH_NONE = 0    #unuse
REG_RCH_CH1  = 1    #CH1


############################################################
##### CH register ##########################################
############################################################
##### ACG: analog coarse gain #####
REG_ACG_X001 = 0
REG_ACG_X002 = 1
REG_ACG_X010 = 2
REG_ACG_X020 = 3

##### ADG: ADG gain #####
REG_ADG_8K  = 0     # 8192 bin
REG_ADG_4K  = 1     # 4096 bin
REG_ADG_2K  = 2     # 2048 bin
REG_ADG_1K  = 3     # 1024 bin
REG_ADG_512 = 4     #  512 bin
REG_ADG_256 = 5     #  256 bin

##### FFD: fast diff #####
REG_FFD_EXT =   0
REG_FFD_20  =  20
REG_FFD_50  =  50
REG_FFD_100 = 100
REG_FFD_200 = 200

##### FFI: fast integral #####
REG_FFI_EXT =   0
REG_FFI_20  =  20
REG_FFI_50  =  50
REG_FFI_100 = 100
REG_FFI_200 = 200

##### PUR: pile up rejector #####
REG_PUR_OFF = 0
REG_PUR_ON  = 1


##### POL: pollarity #####
REG_POL_NEG = 0     #negative
REG_POL_POS = 1     #positive

##### DCG: digital coarce gain #####
REG_DCG_X1   = 0    #   1 times
REG_DCG_X2   = 1    #   2 times
REG_DCG_X4   = 2    #   4 times
REG_DCG_X8   = 3    #   8 times
REG_DCG_X16  = 4    #  16 times
REG_DCG_X32  = 5    #  32 times
REG_DCG_X64  = 6    #  64 times
REG_DCG_X128 = 7    # 128 times

##### DFG: digital fine gain #####
REG_DFG_MIN = (1.0 / 3.0)
REG_DFG_MAX = (1.00)
REG_DFG_COEFFICIENTA = (8193)
REG_DFG_COEFFICIENTB = (-2)

##### TMS: timing select #####
REG_TMS_LET = 0     #leading edge timing
REG_TMS_CFD = 1     #constant fraction discriminator timing

##### CFF: CFD(constant fraction discriminator) function #####
REG_CFF_125 = 1		#x0.125
REG_CFF_250 = 2		#x0.250
REG_CFF_375 = 3		#x0.375
REG_CFF_500 = 4		#x0.500
REG_CFF_625 = 5		#x0.625
REG_CFF_750 = 6		#x0.750
REG_CFF_825 = 7		#x0.825

##### CFD: CFD(constant fraction discriminator) delay #####
REG_CFD_10NS = 0	#
REG_CFD_20NS = 1	#
REG_CFD_30NS = 2	#
REG_CFD_40NS = 3	#
REG_CFD_50NS = 4	#
REG_CFD_60NS = 5	#
REG_CFD_70NS = 6	#
REG_CFD_80NS = 7	#


##### IHW: inhibit width  #####
REG_IHW_MIN =   0   # unit is microseconds
REG_IHW_MAX = 163   # unit is microseconds

##### DIF: coupling #####
REG_DIF_68US = 0	#6.8usec
REG_DIF_22US = 1	#2.2usec
REG_DIF_DC   = 2	#DC


##### FGD: analog fine gain #####
REG_FGD_MIN =   0
REG_FGD_MAX = 255


##### BTS: bit select #####
REG_BTS_AUTO    = 0
REG_BTS_HIGH    = 1     #setting for baseline stabilization at high counting
REG_BTS_DEFAULT = REG_BTS_AUTO

##### BRS: baseline select #####
REG_BRS_GE  = 0     #germanium semiconductor detector(mainly for resistance feedback type preamplifier)
REG_BRS_SDD = 1     #silicon drift detector(mainly for transistor reset type preamplifier)

############################################################
##### HV register ##########################################
############################################################
##### HEN: HV enable #####
REG_HEN_OFF = 0		#off
REG_HEN_ON  = 1		#on

##### HSW: HV sweep time #####
REG_HSW_MIND        = 0
REG_HSW_MAXD        = 366216526
REG_HSW_COEFFICIENT = REG_HSW_MAXD
REG_HSW_COEFFICIENT_1KV =  91554131
REG_HSW_COEFFICIENT_4KV = 366216526

##### HSP: HV bias shutdown plarity #####
REG_HSP_NEG = 0     #negative
REG_HSP_POS = 1     #positive

##### HSJ: HV bias shutdown judge #####
REG_HSJ_MIND =     0
REG_HSJ_MAXD = 65535
REG_HSJ_COEFFICIENT_A = (65535.0 / 48.0)
REG_HSJ_COEFFICIENT_B = (32767.0)

##### HVD: HV output voltage  #####
REG_HVD_MIND =     0
REG_HVD_MAXD = 65535
REG_HVD_COEFFICIENT_A = (4000.0)
REG_HVD_COEFFICIENT_B = (65536.0)

REG_HVD_COEFFICIENT_1KV = (1000.0)
REG_HVD_COEFFICIENT_4KV = (4000.0)

############################################################
##### HV implemented module ################################
############################################################
HVMODULE_IMPLEMENTED_1KV = 1000.0	#implemented 1000V module
HVMODULE_IMPLEMENTED_4KV = 4000.0	#implemented 1000V module


##### :  #####
##### :  #####

