#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys

from . import common
from . import constant
from . import register
from ..common import registerRW

#target register list
TARGET_REGISTER_LIST = [
	"AQS", "MTM", "RLT",
	"ICT", "TCT",
	"ICR", "TCR", "PCR",
	"CLT", "CDT"
]


class Status(registerRW.RegisterRW):
	
	###############################################
	def __init__(self, rbcp = None):
		self.__initializing(True)
		super(Status, self).__init__(rbcp)

		self.mRegisterAttribute = register.Register()
		self.createRegisterInfo(TARGET_REGISTER_LIST, self.mRegisterAttribute)
	
	###############################################
	def __initializing(self, runParent = False):
		if runParent == True:
			super(Status, self)._RegisterRW__initializing()

	###############################################
	def getElapsedSeconds(self, runDeviceRead = False, ipaddr = None, port = None):
		
		secElapsed = None

		if runDeviceRead == True:
			value = self.readStatus("RLT", ipaddr, port)
		else:
			value = self.getStatus("RLT")

		if not value is None:
			secElapsed = common.calcDigit2Seconds(value)

		return secElapsed

	###############################################
	def getElapsedMiiliSeconds(self, runDeviceRead = False, ipaddr = None, port = None):

		msec = None
		sec = self.getElapsedSeconds(runDeviceRead, ipaddr, port)

		if sec is not None:
			msec = int(sec * 1000.0)

		return msec

	###############################################
	def getAcquisitionSeconds(self, runDeviceRead = False, ipaddr = None, port = None):
		
		secElapsed = None

		if runDeviceRead == True:
			value = self.readStatus("MTM", ipaddr, port)
		else:
			value = self.getStatus("MTM")

		if not value is None:
			secElapsed = common.calcDigit2Seconds(value)

		return secElapsed

	###############################################
	def isAcquisitionRuuning(self, runDeviceRead = False, ipaddr = None, port = None):

		isRunning = False

		if runDeviceRead == True:
			self.readStatusAll(ipaddr, port)

		if self.mStatus["AQS,"] == constant.REG_AQS_START:
			mtm = self.getAcquisitionSeconds()
			rlt = self.getElapsedSeconds()

			if rlt < mtm:
				isRunning = True

		return isRunning

	###############################################
	#The status refers to the raed-only register, so writing is not possible.
	#When writing is executed, an exception will be thrown.
	def writeStatus(self, name = None, value = None, ipaddr = None, port = None):
		raise Exception('You cannot write to the status register.')

	###############################################
	#The status refers to the raed-only register, so writing is not possible.
	#When writing is executed, an exception will be thrown.
	def writeStatusAll(self, ipaddr = None, port = None):
		raise Exception('You cannot write to the status register.')


	
