#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys

from . import constant
from . import register
from ..common import registerRW

#target register list
TARGET_REGISTER_LIST = [
	"HST", "HVM", "HIM",
	"HSM", "HSW", "HVD"
]


class StatusHV(registerRW.RegisterRW):
	
	###############################################
	def __init__(self, rbcp = None):
		self.__initializing(True)
		super(StatusHV, self).__init__(rbcp)

		self.mRegisterAttribute = register.Register()
		self.createRegisterInfo(TARGET_REGISTER_LIST, self.mRegisterAttribute)
	
	###############################################
	def __initializing(self, runParent = False):
		if runParent == True:
			super(StatusHV, self)._RegisterRW__initializing()

		self.mStatusClockEmergency = False
		self.mStatusActiveClock    = False
		self.mStatusExtClockBad    = False
		self.mStatusInttClockBad   = False
		self.mStatusEmergency      = False
		self.mStatusBiasShutdown   = False
		self.mStatusPolarity       = False
		self.mStatusOutput         = False
		self.mStatusSweeping       = False
		self.mStatusSweep          = 0
	
	###############################################
	def readStatusAll(self, ipaddr = None, port = None):
		
		result = super(StatusHV, self).readStatusAll(ipaddr, port)

		if result == True:
			self.getStatusDetail()

		return result

	###############################################
	def getOutputVoltage(self, runDeviceRead = False, ipaddr = None, port = None):
		value = self.mStatus["HVM",] if runDeviceRead == False else self.readStatus("HVM", ipaddr, port)
		return self.toPhysicalQuantityHVM(value)

	###############################################
	def getOutputCurrent(self, runDeviceRead = False, ipaddr = None, port = None):
		value = self.mStatus["HIM",] if runDeviceRead == False else self.readStatus("HIM", ipaddr, port)
		return self.toPhysicalQuantityHIM(value)

	###############################################
	def getBiasShutdownVoltage(self, runDeviceRead = False, ipaddr = None, port = None):
		value = self.mStatus["HSM",] if runDeviceRead == False else self.readStatus("HSM", ipaddr, port)
		return self.toPhysicalQuantityHSM(value)
	
	###############################################
	def getSweepTime(self, runDeviceRead = False, ipaddr = None, port = None):
		value = self.mStatus["HSW",] if runDeviceRead == False else self.readStatus("HSW", ipaddr, port)
		return self.toPhysicalQuantityHSW(value)

	###############################################
	def getSettingOutputVoltage(self, runDeviceRead = False, ipaddr = None, port = None):
		value = self.mStatus["HVD",] if runDeviceRead == False else self.readStatus("HVD", ipaddr, port)
		return self.toPhysicalQuantityHVD(value)

	###############################################
	def getStatusDetail(self, runDeviceRead = False, ipaddr = None, port = None):
		status = self.mStatus["HST",] if runDeviceRead == False else self.readStatus("HST", ipaddr, port)
		self.__analyzeStatus(status)
		return None

	###############################################
	def __analyzeStatus(self, status):
		status0 = bytearray(16)

		for i in range(len(status0)):
			status0[i] = (status >> i) & 0x01

		self.mStatusClockEmergency = bool(status0[11])
		self.mStatusActiveClock    = bool(status0[10])
		self.mStatusExtClockBad    = bool(status0[ 9])
		self.mStatusInttClockBad   = bool(status0[ 8])
		self.mStatusEmergency      = bool(status0[ 7])
		self.mStatusBiasShutdown   = bool(status0[ 6])
		self.mStatusPolarity       = bool(status0[ 5])
		self.mStatusOutput         = bool(status0[ 4])
		self.mStatusSweeping       = bool(status0[ 2])
		self.mStatusSweep          = status & 0x0f

	###############################################
	#The status refers to the raed-only register, so writing is not possible.
	#When writing is executed, an exception will be thrown.
	def writeStatus(self, name = None, value = None, ipaddr = None, port = None):
		raise Exception('You cannot write to the status register.')

	###############################################
	#The status refers to the raed-only register, so writing is not possible.
	#When writing is executed, an exception will be thrown.
	def writeStatusAll(self, ipaddr = None, port = None):
		raise Exception('You cannot write to the status register.')

	###############################################
	@staticmethod
	def toPhysicalQuantityHSW(digit):
		Vmin = float(constant.REG_HSW_COEFFICIENT) / float(digit)
		return Vmin

	###############################################
	@staticmethod
	def toPhysicalQuantityHVM(digit):
		volts = None

		if digit >= 32768:
			value0 = float(digit - 32768)
			value1 = 4000.0 / float(0x6666)
			volts = value0 * value1

		return volts

	###############################################
	@staticmethod
	def toPhysicalQuantityHIM(digit):
		uA = None

		if digit >= 32768:
			value0 = 4000.0 / float(0x7fff)
			value1 = float(digit) * value0
			uA = value1 - 4000.0

		return uA

	###############################################
	@staticmethod
	def toPhysicalQuantityHVD(digit):
		volts = (float(digit) * constant.REG_HVD_COEFFICIENT_A) / constant.REG_HVD_COEFFICIENT_B
		return volts

	###############################################
	@staticmethod
	def toPhysicalQuantityHSM(digit):
		volts = (float(digit) * (48.0 / 65535.0)) - 24.0
		return volts
