#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import time

from ...sitcp import constant


class RegisterRW(object):
	
	###############################################
	def __init__(self, rbcp = None):
		self.__initializing()
		self.mRBCP = rbcp
	
	###############################################
	def __initializing(self):

		self.mRBCP = None
		self.mDestPort = None
		self.mDestIpaddr = None
		self.mLastException = None

		self.mStatus = {}
		self.mRegAttribute = None
		self.mListTargetRegister = []

	###############################################
	def isExistRegister(self, name):
		counts = self.mListTargetRegister.count(name)
		return (counts > 0)

	###############################################
	def createRegisterInfo(self, reglist_target, reg_attb):
		self.mRegAttribute = reg_attb
		self.mListTargetRegister = reglist_target
		self.clearStatus()

	###############################################
	def clearStatus(self):
		for i in range(len(self.mListTargetRegister)):
			key = (self.mListTargetRegister[i],)
			self.mStatus[key] = 0

	###############################################
	def getStatus(self, name):
		return self.mStatus[(name,)]

	###############################################
	def setStatus(self, name, value):
		self.mStatus[(name,)] = value

	###############################################
	def setRBCP(self, rbcp):
		self.mRBCP = rbcp

	###############################################
	def setDestination(self, ipaddr, port = constant.DEFAULT_DEST_PORT):
		self.mDestPort = port
		self.mDestIpaddr = ipaddr

	###############################################
	def getAttribute(self, name):
		return self.mRegAttribute.getAttribute(name)

	###############################################
	def getDestAddress(self, ipaddr, port):
		
		work1, work2 = self.mRBCP.getDestAddress(ipaddr, port)
		destipaddr = ipaddr if not ipaddr is None else work1
		destport   = port   if not port   is None else work2

		return	destipaddr, destport

	###############################################
	def readRegisterByName(self, name, ipaddr = None, port = None):
		reg_attb = self.getAttribute(name)
		value = self.mRBCP.readNbytes(reg_attb.mAddress, reg_attb.mBytes, ipaddr, port)
		return value
		
	###############################################
	def readRegisterSignedByName(self, name, ipaddr = None, port = None):
		reg_attb = self.getAttribute(name)
		value = self.mRBCP.readNbytesSigned(reg_attb.mAddress, reg_attb.mBytes, ipaddr, port)
		return value

	###############################################
	def writeRegisterByName(self, name, value, ipaddr = None, port = None):
		reg_attb = self.getAttribute(name)
		result = self.mRBCP.writeNbytes(reg_attb.mAddress, value, reg_attb.mBytes, ipaddr, port)
		return result

	###############################################
	def writeRegisterNtimes(self, name, value_list, intervalms = 0, ipaddr = None, port = None):

		waittm = float(intervalms) / 1000.0		# --> seconds

		for i in range(len(value_list)):
			result = self.writeRegisterByName(name, value_list[i], ipaddr, port)

			if result != True:
				break

			if waittm > 0.0:
				time.sleep(waittm)
			
		return result

	###############################################
	def readStatus(self, name, ipaddr = None, port = None):

		value = None

		try:
			dest_ipaddr, dest_port = self.getDestAddress(ipaddr, port)
			attb  = self.mRegAttribute.getAttribute(name)

			wvalue0 = self.mStatus[name,]

			if isinstance(wvalue0, list) == True:		#list type
				value = []
				wvalue1 = None

				for i in range(len(wvalue0)):
					address = attb.mAddress + (i * attb.mBytes)
					wvalue1 = self.mRBCP.readNbytes(address, attb.mBytes, dest_ipaddr, dest_port)

					if wvalue1 is None:
						break

					value.append(wvalue1)

				if wvalue1 is None:
					value = None
			else:	#single variable
				value = self.mRBCP.readNbytes(attb.mAddress, attb.mBytes, dest_ipaddr, dest_port)

		except Exception as ex:
			self.mLastException = ex
			value = None

		return value
	
	###############################################
	def readStatusAll(self, ipaddr = None, port = None):
		return self._readStatusAll(self, ipaddr, port)

	###############################################
	def writeStatus(self, name, value, ipaddr = None, port = None):

		result = True

		try:
			dest_ipaddr, dest_port = self.getDestAddress(ipaddr, port)
			attb  = self.mRegAttribute.getAttribute(name)

			if isinstance(value, list) == True:		#list type
				valueList = value
			else:	#single variable
				valueList = [value]

			for i in range(len(valueList)):
				address = attb.mAddress + (i * attb.mBytes)
				#result = self.mRBCP.writeNbytes(attb.mAddress, valueList[i], attb.mBytes, dest_ipaddr, dest_port)
				result = self.mRBCP.writeNbytes(address, valueList[i], attb.mBytes, dest_ipaddr, dest_port)

				if result != True:
					break
		except Exception as ex:
			self.mLastException = ex
			result = False

		return result

	###############################################
	def writeStatusAll(self, ipaddr = None, port = None):
		return self._writeStatusAll(self, ipaddr, port)

	###############################################
	@staticmethod
	def _writeStatusAll(object0, ipaddr = None, port = None):

		result = True

		for name, in object0.mStatus.keys():
			value = object0.mStatus[name,]
			result = object0.writeStatus(name, value, ipaddr, port)

			if result == False:
				break
		
		return result
	
	###############################################
	@staticmethod
	def _readStatusAll(object0, ipaddr = None, port = None):

		result = True

		for name, in object0.mStatus.keys():
			value = object0.readStatus(name, ipaddr, port)
			object0.mStatus[name,] = value
			result = result if value is not None else False
		
		return result

