#!/usr/bin/python
# -*- coding: utf-8 -*-

import struct
import socket

from . import constant
from . import rbcpformat
from ..num import byteorder, bitconverter

#constant
AUTOASSIGN_PORT      = constant.AUTOASSIGN_PORT
AUTOASSIGN_PORT_1ST  = constant.AUTOASSIGN_PORT_1ST
AUTOASSIGN_PORT_LAST = constant.AUTOASSIGN_PORT_LAST

DEFAULT_SRC_IPADDR  = constant.DEFAULT_SRC_IPADDR
DEFAULT_DEST_PORT   = constant.DEFAULT_DEST_PORT
DEFAULT_DEST_IPADDR = constant.DEFAULT_DEST_IPADDR

ATTRIBTYPE_TIMEOUT  = constant.ATTRIBTYPE_TIMEOUT
TIMEOUT_ACKNOWLEDGE = constant.TIMEOUT_ACKNOWLEDGE	#milli seconds


class RBCPRaw(object):

	##############################################################
	def __init__(self, initRun = False, src_port = AUTOASSIGN_PORT, src_ipaddr = DEFAULT_SRC_IPADDR):
		self.__initializing()

		if initRun == True:
			self.bind(src_port, src_ipaddr)

	##############################################################
	def __del__(self):
		if self.mSocket != None:
			self.mSocket.close()

	##############################################################
	def __initializing(self):
		self.mSocket = None
		self.mPortMyside = 0
		self.mDefaultDestPort = None
		self.mDefaultDestIpaddr = None
		self.mSequenceNumTX = 0
		self.mInitialized = False
		self.mRequestRead  = rbcpformat.RBCPFormat()
		self.mRequestWrite = rbcpformat.RBCPFormat()
		self.mAckTimeout = constant.RBCP_ACK_TIMEOUT
		self.mBigendianMachine = byteorder.isBigmachine()
		self.setdefaultdest()

	##############################################################
	def free(self):
		if self.mSocket != None:
			self.mSocket.close()
		self.__initializing()

	##############################################################
	def setdefaultdest(self, dest_ipaddr = DEFAULT_DEST_IPADDR, dest_port = DEFAULT_DEST_PORT):
		self.mDefaultDestPort = dest_port
		self.mDefaultDestIpaddr = dest_ipaddr

	##############################################################
	def isBigEndianMachine(self):
		return self.mBigendianMachine

	##############################################################
	def setAcknowledgeTimeout(self, msec):

		result = False

		try:
			sec = float(msec) / 1000.0
			self.mSocket.settimeout(sec)
			self.mAckTimeout = msec
		except Exception as ex:
			self.mLastException = ex

		return result

	##############################################################
	def bind(self, srcport = AUTOASSIGN_PORT, srcipaddr = DEFAULT_SRC_IPADDR):

		sock = None
		result = False

		try:
			sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

			if srcport > AUTOASSIGN_PORT:
				port1st  = srcport
				portlast = srcport
			else:
				port1st  = AUTOASSIGN_PORT_1ST
				portlast = AUTOASSIGN_PORT_LAST

			for portNum in range(port1st, portlast + 1):
				try:
					sock.bind((srcipaddr, portNum))
					self.mSocket = sock
					self.mPortMyside = portNum
					self.mInitialized = True
					self.setAcknowledgeTimeout(self.mAckTimeout)
					result = True
					break
				except Exception as ex:
					pass

		except Exception as ex:
			self.mLastException = ex

		if result == False and sock != None:
			sock.close()

		return result

	##############################################################
	def __getDestAddress(self, destipaddr = None, destport = None):
		dest_ipaddr = destipaddr
		dest_port  = destport

		if dest_ipaddr is None:
			dest_ipaddr = self.mDefaultDestIpaddr
		if dest_port is None:
			dest_port = self.mDefaultDestPort

		return dest_ipaddr, dest_port

	##############################################################
	def getDestAddress(self, destipaddr = None, destport = None):
		return self.__getDestAddress(destipaddr, destport)

	##############################################################
	def requestReading(self, address, nbytes, destipaddr = None, destport = None):

		rxdata = None

		try:
			dest_ipaddr, dest_port = self.__getDestAddress(destipaddr, destport)
			dest_addr = (dest_ipaddr, dest_port)

			#request send
			req = self.mRequestRead
			txdata = req.generateReadCommand(address, nbytes)
			self.mSocket.sendto(txdata, dest_addr)

			#acknowledge recv
			rxbytes = req.getAcknowledgeBytes(nbytes)
			rxdata0, src_addr = self.mSocket.recvfrom(rxbytes)
			rxdata0 = bytearray(rxdata0)
			ack = rbcpformat.RBCPFormat()
			resultACK = ack.restoreCommand(rxdata0)

			if resultACK == True:
				blnOK = ack.isExpectAck(req)
				rxdata = ack.mData if blnOK == True else None
			else:
				pass

		except Exception as ex:
			self.mLastException = ex

		return rxdata

	##############################################################
	def requestWriting(self, address, nbytes, data, destipaddr = None, destport = None):
		
		result = False

		try:
			dest_ipaddr, dest_port = self.__getDestAddress(destipaddr, destport)
			dest_addr = (dest_ipaddr, dest_port)

			#request send
			req = self.mRequestWrite
			data0 = bitconverter.copy_bytearray(data, nbytes)
			txdata = req.generateWriteCommand(address, data0)
			self.mSocket.sendto(txdata, dest_addr)

			#acknowledge recv
			rxbytes = req.getAcknowledgeBytes(nbytes)
			rxdata, src_addr = self.mSocket.recvfrom(rxbytes)
			rxdata = bytearray(rxdata)
			ack = rbcpformat.RBCPFormat()
			resultACK = ack.restoreCommand(rxdata)

			if resultACK == True:
				result = ack.isExpectAck(req)

		except Exception as ex:
			self.mLastException = ex

		return result













